package formulario;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.*;

/**
 * <p>The MIDlet application class that we'll run.</p>
 *
 */
public class FormularioMIDlet extends MIDlet {

    /**
     * Back <code>Command</code>
     */
    private static final Command BACK_COMMAND = new Command("Back", Command.BACK, 0);

    /**
     * Done <code>Command</code>
     */
    private static final Command NEW_COMMAND = new Command("New", Command.SCREEN, 2);

    /**
     * Set <code>Command</code>
     */
    private static final Command SET_COMMAND = new Command("Set", Command.OK, 4);

    /**
     * Exit <code>Command</code>
     */
    private static final Command EXIT_COMMAND = new Command("Exit", Command.STOP, 5);
    private static final Command NEXT_COMMAND = new Command("Next", Command.SCREEN, 6);
    private static final Command PREVIOUS_COMMAND = new Command("Previous", Command.SCREEN, 7);

    /**
     * The <code>MIDlet</code>'s display object
     */
    Display display = null;

    /**
     * The Main menu
     */
    private List menu = null;

    /**
     * The <code>Form</code> to display the different Items
     */
    private Form itemsForm = null;

    /**
     * A textual reference to the current menu that is displayed
     * onscreen to
     * allow the <code>ItemCommandListener</code> to decide what
     * action to perform upon
     * execution of a command
     */
    private String currentMenu = null;

    /**
     * The reference to the <code>ItemDatabase</code> that stores
     * the Items data 
     */
    private ItemsDatabase items = null;

    /**
     * The reference to the <code>AlertDatabase</code> that stores the alerts
     */
    private SettingsDatabase settings = null;

    private boolean firstTime;

    /**
     * <p>Default constructor that is called by the application
     * manager to create a new instance of this <code>MIDlet</code> after
     * which the <code>MIDlet</code> enters the <code>Paused</code> state.</p>
     */
    public FormularioMIDlet() throws MIDletStateChangeException {
        synchronized (this) {
            // Open the items file
            items = new ItemsDatabase();

            try {
                items.open("Items");
            } catch (Exception e) {
                try {
                    items.cleanUp("Items");
                } catch (Exception e2) {
                	e2.printStackTrace();
                }
            }

            // Open the Settings file
            settings = new SettingsDatabase();

            try {
                settings.open("Settings");
            } catch (Exception e) {
                try {
                    settings.cleanUp("Settings");
                } catch (Exception e2) {
                	e2.printStackTrace();
                }
            }

            // Create all the menus and forms

            menu = new List("Main Menu", Choice.IMPLICIT);
            menu.append("view items", null);
            menu.append("Edit items", null);
            menu.append("View Settings", null);
            menu.addCommand(EXIT_COMMAND);
            menu.setCommandListener(new ItemCommandListener());

        } // synchronized

        firstTime = true;
    }

    /**
     * <p>This method is invoked when the <code>MIDlet</code>
     * is ready to run and
     * starts/resumes execution after being in the <code>Paused</code>
     * state. The
     * <code>MIDlet</code> acquires any resources it needs,
     * enters the
     * <code>Active</code> state and begins to perform its service,
     * which in this
     * case means it displays the main menu on the screen.</p>
     *
     * @throws <code>MIDletStateChangeException</code> is thrown if the
     *         <code>MIDlet</code> cannot start now but might be able to
     *         start at a later time.
     * @see javax.microedition.midlet.MIDlet
     */
    public void startApp() {

        display = Display.getDisplay(this);

        if (firstTime) {
            mainMenu();
            firstTime = false;
        }
    }

    /**
     * <p>This method is invoked by the application management software when
     * the <code>MIDlet</code> no longer needs to be active.  It is a stop
     * signal for the <code>MIDlet</code> upon which the <code>MIDlet</code>
     * should release any resources which can be re-acquired through the
     * <code>startApp<code> method which will be called upon re-activation of
     * the <code>MIDlet</code>. The <code>MIDlet</code> enters
     * the <code>Paused</code>
     * state upon completion of this method.</p>
     *
     * @see javax.microedition.midlet.MIDlet
     */
    public void pauseApp() {
        synchronized (this) {
            // free memory used by these objects
            display = null;
        } // synchronized
    }


    public void destroyApp(boolean unconditional) throws MIDletStateChangeException {
        // If there is no criteria that will keep us from terminating
        if (unconditional) {
            synchronized (this) {
                if (display == null) {
                    // If display == null, we are not initialized and
                    // we have nothing to destroy
                    return;
                }

                try {
                    items.close();
                    settings.close();
                } catch (Exception e) {
                    // Ignore exception there is no place to report it
                }

                notifyDestroyed();

            } // synchronized
        }
    }


    /**
     * <p>Display a message onscreen for a specified period of time</p>
     *
     * @param message The message to be displayed
     * @param time The delay before the message disappears
     */
    private void error(String message, int time) {
        if (!(display.getCurrent() instanceof Alert)) {
            Alert a = new Alert("Error", message, null, AlertType.ERROR);
            a.setTimeout(time);
            display.setCurrent(a, display.getCurrent());
        }
    }

    /**
     * <p>Display the main menu of the program</p>
     */
    private void mainMenu() {
        display.setCurrent(menu);
        currentMenu = "Main";
    }

    /**
     * <p>Display the item selected on the View menu with all its
     * attributes shown (ie. Name, Price, etc.)</p>
     *
     * @param theData The content of the item to display
     */
    private void displayItem(String theData) {
    	try {
    		RecordEnumeration re = settings.enumerateRecords();
    		
    		int index = theData.indexOf(',');
    		if(index != -1){
    			theData = theData.substring(index+1);
    		}
    		while (re.hasNextElement() ) {
    			String theFields = new String(re.nextRecord());
    			try {
    				String token = null;
    				String title = Setting.getName(theFields);
    				int numChars = Setting.getNumChars(theFields);
    				int contrains = Setting.getContrains(theFields);
    				index = theData.indexOf(',');
    				if(index == -1){
    					token = "";
    				}else {
    					token = theData.substring(0, index).trim();
    				}
					theData = theData.substring(++index);
					if( ! title.startsWith("wrong")){
	    				
						TextField item = new TextField(title,token, numChars , contrains);
						itemsForm.append(item);
					}
    			} catch (NumberFormatException r1) {
    				r1.printStackTrace();
    			} catch (StringIndexOutOfBoundsException r2) {
    				r2.printStackTrace();
    			}
    		}
    	} catch (RecordStoreNotOpenException rsno) {
    		rsno.printStackTrace();
    	} catch (RecordStoreException rs) {
    		rs.printStackTrace();
    	}

    }

    private void viewItems() {

        	itemsForm = new Form("View Items");	

			try {
				RecordEnumeration re = items.enumerateRecords();

				while (re.hasNextElement()) {
					try {
						String theItem = new String(re.nextRecord());
						if(theItem != null){
							try {
								itemsForm.append(Data.getID(theItem));
							} catch (NullPointerException r){
								r.printStackTrace();
							}
						}
					} catch (InvalidRecordIDException r){
						r.printStackTrace();
					} catch (RecordStoreNotOpenException r){
						r.printStackTrace();
					} catch (RecordStoreException r) { 
						r.printStackTrace();
					}

				}
			} catch (RecordStoreNotOpenException rsno) {
				rsno.printStackTrace();
			} catch (NullPointerException r) {
				r.printStackTrace();
			}
            
            itemsForm.addCommand(BACK_COMMAND);
            itemsForm.setCommandListener(new ItemCommandListener());
            display.setCurrent(itemsForm);
            currentMenu = "View";
    
    }

    
    private void editItems(boolean create) {

    	itemsForm = new Form("items");
    	String theItem = null;
    	try {
    		if(create){
    			Integer in = new Integer(items.getNumRecords()+1);
    			items.add(in.toString());
    			items.SetCurrentIndex(in.intValue());
    			theItem = "";
    			}
    		else{ 
    			try {
    			theItem = items.getRecord(items.getcurrentIndex()); 
    			}catch(InvalidRecordIDException e){
        			Integer in = new Integer(items.getNumRecords()+1);
        			items.add(in.toString());
        			items.SetCurrentIndex(in.intValue());
        			theItem = "";
    			}
    			
    			} 

    		Integer total = new Integer(items.getNumRecords());
    		Integer current = new Integer(items.getcurrentIndex());
    		itemsForm.setTitle("Item "+current.toString() + " of "+total.toString());
    		
    		display.setCurrent(itemsForm);
    		
    		displayItem(theItem+",");
    		
    		if( ! total.equals(current) ) {
    			itemsForm.addCommand(NEXT_COMMAND);
    		}
    		if(items.getcurrentIndex() > 1){
    			itemsForm.addCommand(PREVIOUS_COMMAND);
    		}
    		itemsForm.addCommand(SET_COMMAND);


    	} catch (RecordStoreNotOpenException rsno) {
    		rsno.printStackTrace();
    	} catch (RecordStoreException rs) {
    		rs.printStackTrace();
    	}

    	itemsForm.addCommand(BACK_COMMAND);
    	itemsForm.addCommand(NEW_COMMAND);

    	itemsForm.setCommandListener(new ItemCommandListener());
    	display.setCurrent(itemsForm);
    	currentMenu = "Edit";

    }

    private void viewSettings() {

    	itemsForm = new Form("View Settings");	

    	try {
    		RecordEnumeration re = settings.enumerateRecords();
    		
    		while (re.hasNextElement() ) {
    			String theItem = new String(re.nextRecord());
    			try {
    				StringItem it = new StringItem(Setting.getName(theItem),Setting.getProps(theItem));
    				if(! it.getLabel().startsWith("wrong")){
    					itemsForm.append(it);
    				}
    			} catch (NumberFormatException r1) {
    				r1.printStackTrace();
    			} catch (StringIndexOutOfBoundsException r2) {
    				r2.printStackTrace();
    			}
    		}
    	} catch (RecordStoreNotOpenException rsno) {
    		rsno.printStackTrace();
    	} catch (RecordStoreException rs) {
    		rs.printStackTrace();
    	}

    	itemsForm.addCommand(BACK_COMMAND);
    	itemsForm.setCommandListener(new ItemCommandListener());
    	display.setCurrent(itemsForm);
    	currentMenu = "Settings";

    }

    /**
     * <p>Adds a new Item to the database</p>
     * 
     * extracts the item information from the current displayed item
     *
     */
    private boolean addNewItem() {
    	
    	String symbol = null;
    	symbol = itemsForm.getTitle();
    	symbol=symbol.substring(5, symbol.indexOf(' ',5)).trim();
        try {
            // When items.search() returns null, the item doesn't yet exist
            if (items.search(symbol) == null) {
                try {
                    items.add(getItem(symbol));
                    
                } catch (RecordStoreFullException rsf) {
                    error("Database is full.", 2000);
                    return false;
                } catch (RecordStoreException rs) {
                    error("Failed to add " + symbol, 2000);
                    return false;
                } catch (Exception e) {
                    error(e.toString(), 2000);
                    return false;
                }

                // The Item already exists so we'll just update it
            } else {
                try {
                	String data= new String(getItem(symbol).getBytes());
                    items.update(symbol, data.getBytes());
                } catch (RecordStoreFullException rsf) {
                    error("Database is full.", 2000);

                    return false;
                } catch (RecordStoreException rs) {
                    error("Failed to update " + symbol, 2000);

                    return false;
                } catch (Exception e) {
                	e.printStackTrace();
                    error(e.toString(), 2000);

                    return false;
                }
            }
        } catch (RecordStoreException rs) {
            error("Error accessing database.", 2000);

            return false;
        }

        return true;
    }

    private String getItem(String Symbol) throws IndexOutOfBoundsException {
    	TextField data = null;
    	String strValue = null;
    	int i=0;

    	data=(TextField)itemsForm.get(0);
    	strValue=Symbol.concat(", "+data.getString());

    	for(i=1; i < itemsForm.size(); i++){
    		data=(TextField)itemsForm.get(i);
    		strValue=strValue.concat(", "+data.getString());
    	}
    	return strValue;
    }

    
    /**
     * <p>This class is the Listener for ALL of the events that
     * take place during
     * life span of the <code>MIDlet</code>.  It handles <code>Command</code>
     * events and list selections which are the only events that this
     * <code>MIDlet</code> will generate.  In order to determine what to do,
     * the Listener checks the name of the command and the currently displayed
     * menu/list and matches them up to execute the appropriate action.</p>
     * <BR>
     * NOTE: The parseCommandString(String) is only there because the getXXX
     *       methods are missing from the Command class.  When they are added,
     *       this method can be removed and the Command.getLabel() can be used
     */
    private class ItemCommandListener implements CommandListener, Runnable {
    	/** Current command to process. */
    	private Command currentCommand;

    	/** The current command processing thread. */
    	private Thread commandThread;

    	/**
    	 * <p>The method to determine what action to take</p>
    	 *
    	 * @param c The <code>Command</code> object that has been activated
    	 * @param d The <code>Displayable</code> object that the command was
    	 *          associated with
    	 */
    	public void commandAction(Command c, Displayable d) {
    		synchronized (this) {
    			if (commandThread != null) {
    				// process only one command at a time
    				return;
    			}

    			currentCommand = c;
    			commandThread = new Thread(this);
    			commandThread.start();
    		}
    	}

    	/**
    	 * Perform the current command set by the method commandAction.
    	 */
    	public void run() {
    		String type = currentCommand.getLabel();

    		// Main command executed, always show Main Menu regardless of
    		// which screen is showing
    		if (type.equals("Main")) {
    			mainMenu();

    			// Back command executed, check which screen is and move to
    			// the previous one
    		} else if (type.equals("Back")) {
    			// Screens off the Main Menu
    			if (currentMenu.equals("View") ||
    				currentMenu.equals("Edit") || currentMenu.equals("Settings")) {
    				mainMenu();
    			}
    			/*
    			 * OK command executed, perform different actions
    			 * depending on which screen is showing
    			 */
    		} else if (type.equals("Set")) {
    			if (currentMenu.equals("Edit")) {
    				addNewItem();

    			}
    			// Go to next item
    		} else if (type.equals("Next")) {
    			if (currentMenu.equals("Edit")) {
    				items.SetCurrentIndex(items.getcurrentIndex()+1);
    				editItems(false);
    			}

    			// Go to previous item
    		} else if (type.equals("Previous")) {
        			if (currentMenu.equals("Edit")) {
        				items.SetCurrentIndex(items.getcurrentIndex()-1);
        				editItems(false);
        			}
        			// Create a new item at the end of the list
    		} else if (type.equals("New")) {
        			if (currentMenu.equals("Edit")) {
        				editItems(true);
        			}

        			// Exit command executed
        	}else if (type.equals("Exit")) {
    			try {
    				destroyApp(true);
    			} catch (MIDletStateChangeException msce) {
    				mainMenu();
    			}

    			/*
    			 * No command button was pressed but a list selection
    			 * was made
    			 */
    		} else {
    			List shown = (List)display.getCurrent();

    			/*
    			 * if it's a menu not a list of items then we'll
    			 * use a switch to select which action to perform
    			 */
    			if (currentMenu.equals("Main") || currentMenu.equals("Settings") ||
    					currentMenu.equals("AlertMenu")) {
    				switch (shown.getSelectedIndex()) {
    				case 0:

    					// View Items
    					if (currentMenu.equals("Main")) {
    						viewItems();
    					}

    					break;
    				case 1:

    					// Edit items
    					if (currentMenu.equals("Main")) {
    						editItems(false);
    					}

    					break;
    				case 2:

    					// View Settings
    					if (currentMenu.equals("Main")) {
    						viewSettings();
    					}

    					break;

    				default:
    					break;
    				}
    			} 
    		}

    		synchronized (this) {
    			// signal that another command can be processed
    			commandThread = null;
    		}
    	}
    }
}
