
package formulario;

import javax.microedition.lcdui.TextField;


/**
 * <p>This is a utility class that is used to parse data obtained from either
 * the quote server or the database and break it into fields for easy use
 * rather than spreading the parsing code around to a thousand different places
 * through out the program.</p>
 */
public final class Setting {
	
    /**
     * Minimum value
     */
    private static int min;
    /**
     * Maximum Value
     */
    private static int max;
    /**
     * number of chars
     */
    private static int chars;
    
    /**
     * Name of the Item
     */
    private static String name;

    
    /**
     * Type of the Item
     */
    private static String type;

    
    /**
     * Required Flag
     */
    private static boolean required;


    /**
     * <p>Takes a <code>String</code> from the quote server or database and
     * parses the string into each field.  We first have to split it into
     * small strings and then parse each string that should be a number.
     *
     * @param quoteString the <code>String</code> to parse into the fields
     * @throws <code>NumberFormatException</code> is thrown if
     *            data that is not of the correct format is passed in -- where
     *            the number parts of the string cannot be converted to <code>
     *            Integers</code> because there are non-numeric characters
     *            in positions where numbers are expected
     * @throws <code>StringIndexOutOfBoundsException</code> is
     *            thrown if data that is not of the correct format is passed
     *            in -- where the delimiters between the fields are not in
     *            the right spots, or the data is not of the correct length
     */
    
    public static void parse(String quoteString)
        throws NumberFormatException, StringIndexOutOfBoundsException {
        // get our starting index
    	int from=0;
        int to = quoteString.indexOf(',');

        // split the string up into it's fields
        if (to == -1) {
        	name = "wrong setting";
        	type = "";
        	min = 0;
        	max = 0;
        	chars = 0;
        	required=false;
        	return;
        } else {
        	name = quoteString.substring(from, to ).trim();
        	from=to+1;
        	to = quoteString.indexOf(',', from);
        }
        if (to == -1) {
        	name = "wrong type";
        	type = "";
        	min = 0;
        	max = 0;
        	chars = 0;
        	required=false;
        	return;
        } else {
        	type = quoteString.substring(from, to ).trim();
        	from=to+1;
        	to = quoteString.indexOf(',', from);
        }
        if (to == -1) {
        	name = "wrong min";
        	min = 0;
        	max = 0;
        	chars = 0;
        	required=false;
        	return;
        } else {
        	if(to!=from){
        		try { 
        	min = Integer.valueOf(quoteString.substring(from, to ).trim()).intValue();
        		} catch (NumberFormatException r1) {
         		min=0;
         		name = "wrong min";
        	 }
        	} else {min=0;}
        	from=to+1;
        	to = quoteString.indexOf(',', from);
        }
        if (to == -1) {
        	name = "wrong max";
        	max = 0;
        	chars = 0;
        	required=false;
        	return;
        } else {
        	if(to!=from){
        		try {
        	max = Integer.valueOf(quoteString.substring(from, to ).trim()).intValue();
    		} catch (NumberFormatException r1) {
         		max=0;
         		name = "wrong max";
        	 }
        	} else {max=0;}
        	from=to+1;
        	to = quoteString.indexOf(',', from);
        }
        if (to == -1) {
        	name = "wrong nchars";
        	chars = 0;
        	required=false;
        	return;
        } else {
        	if(from!=to){
        		try {
        	chars = Integer.valueOf(quoteString.substring(from, to ).trim()).intValue();
        		} catch (NumberFormatException r1) {
             		chars=0;
             		name = "wrong chars";
            	 }
        	} else {chars=0;}
        	from=to+1;
        }
        if(quoteString.substring(from).trim().equals("required")){
        	required=true;
        }else if (quoteString.substring(from).trim().equals("not_required")){
        	required=false;
        } else {
        	name = "wrong required";
        	required=false;
        	return;
        }
    }


    /**
     * <p>Return the name of the Item</p>
     *
     * @return name  of the Item
     * @param  data <code>String</code> to parse for the field data
     */
    public static String getName(String data) {
        parse(data);
        return name;
    }

    /**
     * <p>Return the type of the Item</p>
     *
     * @return type of the Item
     * @param  data <code>String</code> to parse for the field data
     */
    public static String getType(String data) {
        parse(data);

        return type;
    }
    public static String getProps(String s){
    	parse(s);
    	return "type:"+type+ " Min:"+min+" Max:"+max+"numOfChars:"+chars+" Required:"+required;
    }
    
    public static int getNumChars(String s){
    	parse(s);
    	return chars;
    }
    public static int getContrains(String s){
    	parse(s);
    	if(type.equals("number")){
    		return TextField.NUMERIC;
    	}else 
    		return TextField.ANY;


    }
    
}
