package formulario;

//import java.util.*;

import javax.microedition.rms.*;


/**
 * <p>This class provides an implementation of the <code>Database</code>
 * class specific to alert records.</p>
 */
public class SettingsDatabase extends Database {
    /**
     * Default Constructor
     */
    public SettingsDatabase() {
        rc = new SettingComparator();
    }

    /**
     * <p>Get a <code>RecordEnumeration</code> of records in the database who
     * match the <code>AlertFilter</code> conditions</p>
     *
     * @return <code>RecordEnumeration</code> of all settings that match
     *         the <code>RecordFilter</code>
     * @throws <code>RecordStoreNotOpenException</code> is thrown when
     *         trying to close a <code>RecordStore</code> that is not open
     */
    public synchronized RecordEnumeration enumerateRecords()
        throws RecordStoreNotOpenException {
        return database.enumerateRecords(null, null, true);
    }

    /**
     *  <p>Class to compare two records and see if they are equal</p>
     *
     * @see javax.microedition.rms.RecordComparator
     */
    private class SettingComparator implements RecordComparator {
        /**
         * <p>Checks to see if rec1 matches rec2</p>
         *
         * @return <code>RecordComparator.EQUIVALENT</code> if the records
         *          match or <code>Integer.MAX_VALUE</code> if they don't
         * @param rec1 the data to compare against
         * @param rec2 the data to compare with
         */
        public int compare(byte[] rec1, byte[] rec2) {
     
            String record1 = new String(rec1);
            String record2 = new String(rec2);

            if (record1.equals(record2)) {
                return RecordComparator.EQUIVALENT;
            }

            return Integer.MAX_VALUE;
        }
    }
}