package formulario.load;

import java.io.*;

import java.util.*;

import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.*;

import formulario.SettingsDatabase;



/**
 * Demonstration MIDlet for File Connection API. This MIDlet implements simple
 * file browser for the filesystem available to the J2ME applications.
 *
 */
public class LoadSettings extends MIDlet implements CommandListener {
	
    /* special string denotes upper directory */
    private static final String UP_DIRECTORY = "..";

    /* special string that denotes upper directory accessible by this browser.
     * this virtual directory contains all roots.
     */
    private static final String MEGA_ROOT = "/";

    /* separator string as defined by FC specification */
    private static final String SEP_STR = "/";

    /* separator character as defined by FC specification */
    private static final char SEP = '/';
    private String currDirName;
    private Command view = new Command("View", Command.ITEM, 1);

    //add delete file functionality
    private Command load = new Command("Load", Command.ITEM, 2);
    private Command back = new Command("Back", Command.BACK, 2);
    private Command exit = new Command("Exit", Command.EXIT, 3);
    private Image dirIcon;
    private Image fileIcon;
    private Image[] iconList;
    
    private SettingsDatabase settings = null;


    public LoadSettings() {
        currDirName = MEGA_ROOT;

        try {
            dirIcon = Image.createImage("/icons/dir.png");
        } catch (IOException e) {
            dirIcon = null;
        }

        try {
            fileIcon = Image.createImage("/icons/file.png");
        } catch (IOException e) {
            fileIcon = null;
        }

        iconList = new Image[] { fileIcon, dirIcon };
    }

    public void startApp() {
        try {
            showCurrDir();
        } catch (SecurityException e) {
            Alert alert =
                new Alert("Error", "You are not authorized to access the restricted API", null,
                    AlertType.ERROR);
            alert.setTimeout(Alert.FOREVER);

            Form form = new Form("Cannot access FileConnection");
            form.append(new StringItem(null,
                    "You cannot run this MIDlet with the current permissions. " +
                    "Sign the MIDlet suite, or run it in a different security domain"));
            form.addCommand(exit);
            form.setCommandListener(this);
            Display.getDisplay(this).setCurrent(alert, form);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean cond) {
        notifyDestroyed();
    }

    public void commandAction(Command c, Displayable d) {
        if (c == view) {
            List curr = (List)d;
            final String currFile = curr.getString(curr.getSelectedIndex());
            new Thread(new Runnable() {
                    public void run() {
                        if (currFile.endsWith(SEP_STR) || currFile.equals(UP_DIRECTORY)) {
                            traverseDirectory(currFile);
                        } else {
                            // Show file contents
                            showFile(currFile);
                        }
                    }
                }).start();
        } else if (c == load) {
            List curr = (List)d;
            final String currFile = curr.getString(curr.getSelectedIndex());
            new Thread(new Runnable() {
                public void run() {
                    if (currFile.endsWith(SEP_STR) || currFile.equals(UP_DIRECTORY)) {
                        traverseDirectory(currFile);
                    } else {
                        // load Settings file
                        loadSettings(currFile);
                    }
                }
            }).start();

        }else if (c == back) {
            showCurrDir();
        } else if (c == exit) {
            destroyApp(false);
        } 
    }


    /**
     * Show file list in the current directory .
     */
    void showCurrDir() {
        Enumeration e;
        FileConnection currDir = null;
        List browser;

        try {
            if (MEGA_ROOT.equals(currDirName)) {
                e = FileSystemRegistry.listRoots();
                browser = new List(currDirName, List.IMPLICIT);
            } else {
                currDir = (FileConnection)Connector.open("file://localhost/" + currDirName);
                e = currDir.list();
                browser = new List(currDirName, List.IMPLICIT);
                // not root - draw UP_DIRECTORY
                browser.append(UP_DIRECTORY, dirIcon);
            }

            while (e.hasMoreElements()) {
                String fileName = (String)e.nextElement();

                if (fileName.charAt(fileName.length() - 1) == SEP) {
                    // This is directory
                    browser.append(fileName, dirIcon);
                } else {
                    // this is regular file
                    browser.append(fileName, fileIcon);
                }
            }

            browser.setSelectCommand(view);
            if (!MEGA_ROOT.equals(currDirName)) {
                browser.addCommand(load);
            }

            browser.addCommand(exit);

            browser.setCommandListener(this);

            if (currDir != null) {
                currDir.close();
            }

            Display.getDisplay(this).setCurrent(browser);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    void traverseDirectory(String fileName) {
        /* In case of directory just change the current directory
         * and show it
         */
        if (currDirName.equals(MEGA_ROOT)) {
            if (fileName.equals(UP_DIRECTORY)) {
                // can not go up from MEGA_ROOT
                return;
            }

            currDirName = fileName;
        } else if (fileName.equals(UP_DIRECTORY)) {
            // Go up one directory
            int i = currDirName.lastIndexOf(SEP, currDirName.length() - 2);

            if (i != -1) {
                currDirName = currDirName.substring(0, i + 1);
            } else {
                currDirName = MEGA_ROOT;
            }
        } else {
            currDirName = currDirName + fileName;
        }

        showCurrDir();
    }

    void showFile(String fileName) {
        try {
            FileConnection fc =
                (FileConnection)Connector.open("file://localhost/" + currDirName + fileName);

            if (!fc.exists()) {
                throw new IOException("File does not exists");
            }

            InputStream fis = fc.openInputStream();
            byte[] b = new byte[1024];

            int length = fis.read(b, 0, 1024);

            fis.close();
            fc.close();

            TextBox viewer =
                new TextBox("View File: " + fileName, null, 1024,
                    TextField.ANY | TextField.UNEDITABLE);

            viewer.addCommand(back);
            viewer.addCommand(exit);
            viewer.setCommandListener(this);

            if (length > 0) {
                viewer.setString(new String(b, 0, length));
            }

            Display.getDisplay(this).setCurrent(viewer);
        } catch (Exception e) {
            Alert alert =
                new Alert("Error!",
                    "Can not access file " + fileName + " in directory " + currDirName +
                    "\nException: " + e.getMessage(), null, AlertType.ERROR);
            alert.setTimeout(Alert.FOREVER);
            Display.getDisplay(this).setCurrent(alert);
        }
    }
    void loadSettings(String fileName) {
		System.out.println("abriendo archivo "+ fileName);

    	try {
    		if (fileName.equals(UP_DIRECTORY)) {
    			return;
    		}
    		System.out.println("no es UP_directory");
    		FileConnection fc =
    			(FileConnection)Connector.open("file://localhost/" + currDirName + fileName);
    		System.out.println("creado fc");
    		if (!fc.exists()) {
    			throw new IOException("File does not exists");
    		}
    		System.out.println("no existe el archivo");
    		InputStreamReader reader = new InputStreamReader(fc.openInputStream());
    		System.out.println("archivo abierto");
    		
    		// Open the Settings database and clean it
    		settings = new SettingsDatabase();

    		try {
    			settings.open("Settings");
    			System.out.println("settings.open()");
    			
    	    	try {
    	    		
    	    		RecordEnumeration re = settings.enumerateRecords();
    	    		System.out.println("hay "+re.numRecords()+"elementos");
    	    		while (re.hasNextElement() ) {
    	    			String theItem = new String(re.nextRecord());
    	    			settings.delete(theItem);
    	    			System.out.println("hay ="+settings.getNumRecords()+" re="+re.numRecords());
    	    		}
    	    	} catch (RecordStoreNotOpenException rsno) {
    	    		rsno.printStackTrace();
    	    	} catch (RecordStoreException rs) {
    	    		rs.printStackTrace();
    	    	}

    			
    			
    		} catch (Exception e2) {System.out.println("settings.open() catch");
    			e2.printStackTrace();
    			try {
    				System.out.println("stry cleanup");
    				settings.cleanUp("Settings");
    			} catch (Exception e3) {
    				e3.printStackTrace();
    			}
    		}


    		System.out.println("db abierta");
    		String line = null;
    		// Read a single line from the file. null represents the EOF.
    		while ((line = readLine(reader)) != null) {
    			try {
    				settings.add(line);
    			}catch (RecordStoreNotOpenException e){
    				e.printStackTrace();
    			}catch (RecordStoreFullException e2){
    				e2.printStackTrace();
    			}catch (RecordStoreException e3){
    				e3.printStackTrace();
    			}
    			System.out.println("cargado ="+ line+"hay ="+settings.getNumRecords());
    		}
    		System.out.println("close");
    		reader.close();
    		fc.close();
    		settings.close();

    		Alert alert = new Alert("", "\n\n\n   Settings Saved!", null, AlertType.INFO);
    		alert.setTimeout(Alert.FOREVER);
    		Display.getDisplay(this).setCurrent(alert);

    	} catch (Exception e) {
    		e.printStackTrace();
    		Alert alert =
    			new Alert("Error!",
    					"Can not access file " + fileName + " in directory " + currDirName +
    					"\nException: " + e.getMessage(), null, AlertType.ERROR);
    		alert.setTimeout(Alert.FOREVER);
    		Display.getDisplay(this).setCurrent(alert);
    	}
    }
 
    /**
     * Reads a single line using the specified reader.
     * @throws java.io.IOException if an exception occurs when reading the
     * line
     */
    private String readLine(InputStreamReader reader) throws IOException {
        // Test whether the end of file has been reached. If so, return null.
        int readChar = reader.read();
        if (readChar == -1) {
            return null;
        }
        StringBuffer string = new StringBuffer("");
        // Read until end of file or new line
        while (readChar != -1 && readChar != '\n') {
            // Append the read character to the string. Some operating systems
            // such as Microsoft Windows prepend newline character ('\n') with
            // carriage return ('\r'). This is part of the newline character
            // and therefore an exception that should not be appended to the
            // string.
            if (readChar != '\r') {
                string.append((char)readChar);
            }
            // Read the next character
            readChar = reader.read();
        }
        return string.toString();
    }
}
